import { FC, useCallback, useEffect, useState } from 'react'
import AnimatedPageContainer from '../AnimatedPageContainer';
import { TypeEditor } from './type-editor';
import { useObjectState, useObjectStateWrapper } from '../../hooks/useObjectState';
import { FieldTypes, Schema, Template, TypeType } from '../../types/schema';
import { useInput } from '../../hooks/useInput';
import { useRecoilState } from 'recoil';
import { SchemaEditAtom } from '../../recoil/atoms/schema';
import { GameSystemsService } from '../../services/game-systems';
import { SchemaViewer } from './schema-viewer';
import { TemplateEditor } from './template-editor';


export const SchemaBuilder: FC = () => {
  const [schema, setSchema] = useRecoilState(SchemaEditAtom);
  const { update: updateSchema } = useObjectStateWrapper<Schema>(schema, setSchema);

  const { value: typeName, bind: bindTypeName, reset: resetTypeName } = useInput('');

  const [pageNumber, setPageNumber] = useState(0);

  const [lastSaved, setLastSaved] = useState(schema);

  const fetchSchema = useCallback(async () => {
    const result = await GameSystemsService.getSchema('286f4c18-d280-444b-8d7e-9a3dd09f64ef')
    if (result.status !== 200) return;
    const fetchedSchema = await result.json();
    // if (fetchedSchema.name === schema.name) return;
    if (!fetchedSchema.templates) fetchedSchema.templates = {}
    setSchema(fetchedSchema);
    setLastSaved(fetchedSchema);
  }, [setSchema])

  useEffect(() => {
    fetchSchema();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [])

  const [selectedType, setSelectedType] = useState('');

  const saveType = useCallback((name: string, type: TypeType) => {
    updateSchema(e => ({
      types: {
        ...e.types,
        [name]: type
      }
    }));
    resetTypeName();
    setPageNumber(0);
    setSelectedType('');
  }, [resetTypeName, updateSchema]);

  const saveSchema = useCallback(() => {
    GameSystemsService.saveSchema(schema);
    setLastSaved(schema);
  }, [schema])

  const selectTypeForEdit = useCallback((typeKey: string) => {
    setSelectedType(typeKey);
    setPageNumber(1);
  }, [])

  const { value: templateName, bind: bindTemplateName, reset: resetTemplateName } = useInput('');
  const addTemplate = useCallback(() => {
    updateSchema(s => ({
      templates: {
        ...s.templates,
        [templateName]: {
          publishable: false,
          type: FieldTypes.any
        }
      }
    }));
    resetTemplateName();
  }, [resetTemplateName, templateName, updateSchema])

  const updateTemplate = useCallback((key: string, template: Template) => {
    updateSchema(s => ({
      templates: {
        ...s.templates,
        [key]: template
      }
    }))
  }, [updateSchema])

  return (
    <div className="container flex gap-4 p-8">
      <div className="panel w-2/3 h-full">
        <p className="subheader">Add a template</p>
        <input type="text" {...bindTemplateName} />
        <button onClick={addTemplate} disabled={!templateName}>Add</button>
        <ul>
          {Object.entries(schema.templates).map(([templateKey, template]) => (
            <TemplateEditor templateKey={templateKey} template={template} update={updateTemplate} />
          ))}
        </ul>
        <AnimatedPageContainer currentPage={pageNumber}>
          <div>
            <p className="subheader">Add a type</p>
            <input type="text" {...bindTypeName} />
            <button className="interactive" disabled={!typeName} onClick={() => setPageNumber(1)}>Configure</button>
          </div>
          <TypeEditor name={selectedType || typeName} saveType={saveType} type={selectedType ? schema.types[selectedType as keyof typeof schema.types] : undefined} />
        </AnimatedPageContainer>
      </div>
      <div className="panel basis-1/3">
        <SchemaViewer schema={schema} onTypeClick={selectTypeForEdit} />
        <div className="flex gap-2 mt-2">
          <button
            onClick={saveSchema}
            disabled={lastSaved === schema}
          >
            Save Schema
          </button>
          <button
            className="bg-red-800"
            onClick={() => setSchema(lastSaved)}
            disabled={lastSaved === schema}
          >
            Discard Changes
          </button>
        </div>
      </div>
    </div>
  )
}